#!/usr/bin/env python

import os
import httplib
import base64
from selenium import webdriver
from robot.libraries.BuiltIn import BuiltIn
from applitools import logger
from applitools.logger import StdoutLogger
from applitools.logger import FileLogger
from applitools.geometry import Region
from applitools.eyes import Eyes, BatchInfo
from applitools.selenium.webelement import EyesWebElement
from robot.api import logger as loggerRobot

from EyesLibrary.resources import utils, variables


class SessionKeywords(object):

    library_arguments = {}

    def open_eyes_session(
        self,
        apikey=None,
        appname=None,
        testname=None,
        library=None,
        width=None,
        height=None,
        osname=None,
        browsername=None,
        matchlevel=None,
        enable_eyes_log=None,
        enable_http_debug_log=None,
        baselinename=None,
        batchname=None,
        branchname=None,
        parentbranch=None,
    ):
        """
        Starts a session with the Applitools Eyes Website. See https://eyes.applitools.com/app/sessions/
        Some of the following arguments may also be defined on library import.~
        See `Before running tests` or `Importing`

                | *Arguments*                           | *Description*                                                                                               |
                |  API Key (string)                     | User's Applitools Eyes key.                                                                                 |
                |  Application Name (string)            | The name of the application under test.                                                                     |
                |  Test Name (string)                   | The test name.                                                                                              |  
                |  Library (default=SeleniumLibrary)    | Library to test (Either SeleniumLibrary or AppiumLibrary)                                                   |
                |  (Optional) Width (int)               | The width of the browser window e.g. 1280                                                                   |
                |  (Optional) Height (int)              | The height of the browser window e.g. 1000                                                                  |
                |  (Optional) Operating System (string) | The operating system of the test, can be used to override the OS name to allow cross OS verfication         |
                |  (Optional) Browser Name (string)     | The browser name for the test, can be used to override the browser name to allow cross browser verification  |
                |  (Optional) Match Level (string)      | The match level for the comparison - can be STRICT, LAYOUT, CONTENT or EXACT                                    |
                |  Enable Eyes Log (default=False)     | The Eyes logs will not be included by default. To activate, pass 'True' in the variable.                    |
                |  Enable HTTP Debug Log (default=False)       | The HTTP Debug logs will not be included by default. To activate, pass 'True' in the variable.              |
                |  Baseline Name (default=None)  | Name of the branch where the baseline reference will be taken from and where new and accepted steps will be saved to.                                                  |
                |  Batch Name (default=None)        | The name of the batch                      |
                |  Branch Name (default=None)  | The branch to use to check test                                                                             |
                |  Parent Branch (default=None)        | Parent Branch to base the new Branch on                                                                     |
        Creates an instance of the AppiumLibrary or SeleniumLibrary webdriver, given the library argument.

        Defines a global driver and sets the webdriver to the global driver.

        Checks if there has been a width or height value passed in.
        If there no are values passed in, eyes calls the method open without the width and height values.
        Otherwise, Eyes calls open with the width and height values defined.

        It is mandatory to define at least the Application Name, Test Name and API Key, either through this keyword or when importing the library. 

        *Note:* When opening the session on a mobile browser or hybrid app, the context must be set to WEBVIEW in order to retrieve the correct viewport size. Geolocation of the device may have to be set after switching context.

        *Example:*                                                                                                                                                                                                                               
                | Open Eyes Session  |   YourApplitoolsKey  | Eyes_AppName |  Eyes_TestName | SeleniumLibrary |  1024  |  768  |  OSOverrideName  |  BrowserOverrideName  |  LAYOUT   |  True  |  True  |  BranchName   |  ParentBranch   |
        """

        if appname is None:
            appname = self.library_arguments["appname"]
        if testname is None:
            testname = self.library_arguments["testname"]
        if apikey is None:
            apikey = self.library_arguments["apikey"]
        if library is None:
            library = self.library_arguments["library"]
        if osname is None:
            osname = self.library_arguments["osname"]
        if browsername is None:
            browsername = self.library_arguments["browsername"]
        if matchlevel is None:
            matchlevel = self.library_arguments["matchlevel"]
        if enable_eyes_log is None:
            enable_eyes_log = self.library_arguments["enable_eyes_log"]

        variables.eyes = Eyes()
        variables.eyes.api_key = apikey

        try:
            libraryInstance = BuiltIn().get_library_instance(library)

            if library == "AppiumLibrary":
                driver = libraryInstance._current_application()
            else:
                driver = libraryInstance._current_browser()
        except RuntimeError:
            raise Exception("%s instance not found" % library)

        utils.manage_logging(enable_eyes_log, enable_http_debug_log)

        if osname is not None:
            variables.eyes.host_os = osname  # (str)
        if browsername is not None:
            variables.eyes.host_app = browsername  # (str)
        if baselinename is not None:
            variables.eyes.baseline_branch_name = baselinename  # (str)
        if batchname is not None:
            batch = BatchInfo(batchname)
            variables.eyes.batch = batch
        if matchlevel is not None:
            variables.eyes.match_level = utils.get_match_level(matchlevel)
        if parentbranch is not None:
            variables.eyes.parent_branch_name = parentbranch  # (str)
        if branchname is not None:
            variables.eyes.branch_name = branchname  # (str)

        if width is None and height is None:
            variables.driver = variables.eyes.open(driver, appname, testname)
        else:
            intwidth = int(width)
            intheight = int(height)

            variables.driver = variables.eyes.open(
                driver, appname, testname, {"width": intwidth, "height": intheight}
            )

    def close_eyes_session(self, enable_eyes_log=False, enable_http_debug_log=False):
        """
        Closes a session and returns the results of the session.
        If a test is running, aborts it. Otherwise, does nothing.

                |  *Arguments*                      | *Description*                                                                                   |
                |  Enable Eyes Log (default=False) | The Eyes logs will not be included by default. To activate, pass 'True' in the variable.        |
                |  Enable HTTP Debug Log (default=False)   | The HTTP Debug logs will not be included by default. To activate, pass 'True' in the variable.  |

        *Example:*
            | Close Eyes Session    |    True   |   True    |                                 
        """
        utils.manage_logging(enable_eyes_log, enable_http_debug_log)

        variables.eyes.close()
        variables.eyes.abort_if_not_closed()

    def eyes_session_is_open(self):
        """
        Returns True if an Applitools Eyes session is currently running, otherwise it will return False.

        *Example:*
            | ${isOpen}=        |  Eyes Session Is Open     |                    
        """
        return variables.eyes.is_open



import httplib
from applitools.core import logger, StdoutLogger
from applitools.eyes import MatchLevel
from applitools.core import EyesIllegalArgument
from selenium.webdriver.common.by import By
from selenium.common.exceptions import InvalidElementStateException



def get_match_level(matchlevel):

    selected_match_level = None

    if matchlevel.upper() == "STRICT":
        selected_match_level = MatchLevel.STRICT
    elif matchlevel.upper() == "CONTENT":
        selected_match_level = MatchLevel.CONTENT
    elif matchlevel.upper() == "LAYOUT":
        selected_match_level = MatchLevel.LAYOUT
    elif matchlevel.upper() == "EXACT":
        selected_match_level = MatchLevel.EXACT
    else:
        raise EyesIllegalArgument(
            "Please select a valid match level: Strict, Content, Layout, Exact"
        )

    return selected_match_level


def get_selector_strategy(selector):

    selected_strategy = None

    if selector.upper() == "CSS SELECTOR":
        selected_strategy = By.CSS_SELECTOR
    elif selector.upper() == "XPATH":
        selected_strategy = By.XPATH
    elif selector.upper() == "ID":
        selected_strategy = By.ID
    elif selector.upper() == "LINK TEXT":
        selected_strategy = By.LINK_TEXT
    elif selector.upper() == "PARTIAL LINK TEXT":
        selected_strategy = By.PARTIAL_LINK_TEXT
    elif selector.upper() == "NAME":
        selected_strategy = By.NAME
    elif selector.upper() == "TAG NAME":
        selected_strategy = By.TAG_NAME
    elif selector.upper() == "CLASS NAME":
        selected_strategy = By.CLASS_NAME
    else:
        raise InvalidElementStateException(
            "Please select a valid selector: CSS SELECTOR, XPATH, ID, LINK TEXT, PARTIAL LINK TEXT, NAME, TAG NAME, CLASS NAME"
        )

    return selected_strategy


def manage_logging(enable_eyes_log, enable_http_debug_log):
    
    if enable_eyes_log is True:
        logger.set_logger(StdoutLogger())
        logger.open_()
    if enable_http_debug_log is True:
        httplib.HTTPConnection.debuglevel = 1



#!/usr/bin/env python

import os
import httplib
import base64
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.common.exceptions import InvalidElementStateException
from robot.libraries.BuiltIn import BuiltIn
from applitools.core import logger

# from applitools.logger import StdoutLogger
# from applitools.logger import FileLogger
from applitools.geometry import Region
from applitools.eyes import Eyes, BatchInfo
from applitools.selenium.webelement import EyesWebElement
from .session import SessionKeywords
from robot.api import logger as loggerRobot

from EyesLibrary.resources import variables, utils


class CheckKeywords:
    def check_eyes_window(
        self,
        name,
        force_full_page_screenshot=False,
        enable_eyes_log=False,
        enable_http_debug_log=False,
    ):
        """
        Takes a snapshot from the browser using the web driver and matches it with
        the expected output.

                |  *Arguments*                                  | *Description*                                                                                   |
                |  Name (string)                                | Name that will be given to region in Eyes.                                                      |
                |  Force Full Page Screenshot (default=False)   | Will force the browser to take a screenshot of whole page.                                      |
                |  Enable Eyes Log (default=False)             | The Eyes logs will not be included by default. To activate, pass 'True' in the variable.        |
                |  Enable HTTP Debug Log (default=False)               | The HTTP Debug logs will not be included by default. To activate, pass 'True' in the variable.  |

        *Example:*
            | Check Eyes Window  |  Google Homepage   |  True   |  True  |   True   |

        """
        utils.manage_logging(enable_eyes_log, enable_http_debug_log)

        variables.eyes.force_full_page_screenshot = force_full_page_screenshot
        variables.eyes.check_window(name)

    def check_eyes_region(
        self,
        left,
        top,
        width,
        height,
        name,
        enable_eyes_log=False,
        enable_http_debug_log=False,
    ):
        """
        Takes a snapshot of the given region from the browser using a Region object (identified by left, top, width, height)
        and matches it with the expected output.

        The width and the height cannot be greater than the width and the height specified in the open_eyes_session keyword.

                |  *Arguments*                       | *Description*                                         |
                |  Left (float)                      | The left coordinate of the region that is tested e.g. 100                                      |
                |  Top (float)                       | The top coordinate of the region that is tested e.g. 150                                       |
                |  Width (float)                     | The width of the region that is tested e.g. 500                                                |
                |  Height (float)                    | The height of the region that is tested e.g. 120                                               |
                |  Name (string)                     | Name that will be given to region in Eyes.                                                     |
                |  Enable Eyes Log (default=False)  | The Eyes logs will not be included by default. To activate, pass 'True' in the variable.       |
                |  Enable HTTP Debug Log (default=False)    | The HTTP Debug logs will not be included by default. To activate, pass 'True' in the variable. |

        *Example:*            
            | Check Eyes Region  |  100   | 150   | 500   |  120  |  Google Logo   |  True  | True  |

        """

        utils.manage_logging(enable_eyes_log, enable_http_debug_log)

        region = Region(float(left), float(top), float(width), float(height))
        variables.eyes.check_region(region, name)

    def check_eyes_region_by_element(
        self, element, name, enable_eyes_log=False, enable_http_debug_log=False
    ):
        """
        Takes a snapshot of the region of the given element from the browser using the web driver. Not available to mobile native apps.

                |  *Arguments*                      | *Description*                                      |
                |  Element (WebElement)             | The Web Element to be checked. See `Get Element`   |
                |  Name (string)                    | Name that will be given to region in Eyes.                                                                           |
                |  Enable Eyes Log (default=False) | The Eyes logs will not be included by default. To activate, pass 'True' in the variable.                             |
                |  Enable HTTP Debug Log (default=False)   | The HTTP Debug logs will not be included by default. To activate, pass 'True' in the variable.                       |

        *Example:*
            | ${element}=    Get Element   |    //*[@id="hplogo"]  |
            | Check Eyes Region By Element  |  ${element}     |  ElementName    |   True  | True  |    

        *Note (Safari on mobile):* 
        When checking an element, provide osname=iOS and browsername=Safari on `Open Eyes Session`.
        Due to an issue regarding the height of the address bar not being taken into account when the screenshot is taken, a temporary workaround is in place.
        In order to screenshot the correct element, it is added the value of 71 to the y coordinate of the element.

        """

        utils.manage_logging(enable_eyes_log, enable_http_debug_log)

        if not isinstance(element, EyesWebElement):
            element = EyesWebElement(element, variables.driver)

        # Temporary workaround in order to capture the correct element on Safari
        # Element coordinate y doesn't take the address bar height into consideration, so it has to be added
        # Current address bar height: 71
        if variables.eyes.host_app == "Safari" and variables.eyes.host_os == "iOS":
            location = element.location
            size = element.size

            variables.eyes.check_region(
                Region(
                    location.__getitem__("x"),
                    location.__getitem__("y") + 71,
                    size.__getitem__("width"),
                    size.__getitem__("height"),
                )
            )
        else:
            variables.eyes.check_region_by_element(element, name)

    def check_eyes_region_by_selector(
        self,
        value,
        name,
        selector="id",
        enable_eyes_log=False,
        enable_http_debug_log=False,
    ):
        """
        Takes a snapshot of the region of the element found by calling find_element(by, value) from the browser using the web driver
        and matches it with the expected output. With a choice from eight selectors, listed below to check by.

        Not available to mobile native apps.

                |  *Arguments*                      | *Description*                                                                              |
                |  Value (string)                   | The specific value of the selector. e.g. a CSS SELECTOR value .first.expanded.dropdown                                                                                |
                |  Name (string)                    | Name that will be given to region in Eyes.                                                                                                                            |
                |  Selector (default=id)            | This will decide what element will be located. The supported selectors include: CSS SELECTOR, XPATH, ID, LINK TEXT, PARTIAL LINK TEXT, NAME, TAG NAME, CLASS NAME.    |
                |  Enable Eyes Log (default=False) | The Eyes logs will not be included by default. To activate, pass 'True' in the variable.                                                                              |
                |  Enable HTTP Debug Log (default=False)   | The HTTP Debug logs will not be included by default. To activate, pass 'True' in the variable.                                                                        |

        *Example:*
            | Check Eyes Region By Selector |    .first.expanded.dropdown |  CssElement         |     CSS SELECTOR             |

        *Note (Safari on mobile):* 
        When checking an element, provide osname=iOS and browsername=Safari on `Open Eyes Session`.
        Due to an issue regarding the height of the address bar not being taken into account when the screenshot is taken, a temporary workaround is in place.
        In order to screenshot the correct element, it is added the value of 71 to the y coordinate of the element.

        """
        utils.manage_logging(enable_eyes_log, enable_http_debug_log)

        selector_strategy = utils.get_selector_strategy(selector)

        # Temporary workaround in order to capture the correct element on Safari
        # Element coordinate y doesn't take the address bar height into consideration, so it has to be added
        # Current address bar height: 71
        if variables.eyes.host_app == "Safari" and variables.eyes.host_os == "iOS":
            element = variables.driver.find_element(selector_strategy, value)
            location = element.location
            size = element.size

            variables.eyes.check_region(
                Region(
                    location.__getitem__("x"),
                    location.__getitem__("y") + 71,
                    size.__getitem__("width"),
                    size.__getitem__("height"),
                )
            )
        else:
            variables.eyes.check_region_by_selector(selector_strategy, value, name)



#!/usr/bin/env python

from .keywords import SessionKeywords, CheckKeywords
from .resources import variables
from .version import VERSION

_version_ = VERSION


class EyesLibrary(SessionKeywords, CheckKeywords):
    """
    EyesLibrary is a visual verification library for Robot Framework that leverages
    the Eyes-Selenium and Selenium/Appium libraries.

    *Before running tests*

     In order to run EyesLibrary and return results, you have to create a free account https://applitools.com/sign-up/ with Applitools.
    You can retrieve your API key from the applitools website and that will need to be passed in your Open Eyes Session keyword.

    Prior to running tests, EyesLibrary must first be imported into your Robot test suite.

    Example:
        | Library | EyesLibrary | 

    You may define the following arguments when importing the library (You may also define them on `Open Eyes Session`):
    - API Key (apikey)
    - Application Name (appname)
    - Test Name (testname)
    - Library - SeleniumLibrary or AppiumLibrary (library)
    - Match Level - Strict, Exact, Content or Layout (matchlevel)
    - Enable Eyes Logs (enable_eyes_log)
    - OS Name (osname)
    - Browser Name (browsername)

    Example:
        | Library | EyesLibrary | ApiKey | AppName | TestName | SeleniumLibrary | Layout | True | Windows | Firefox |
        
    *Writing tests*

    When writing the tests, the following structure must be adopted:

    | *Open Eyes Session* |
    | A browser or application must be running when opening the session. To open a browser/application, consult the documentation for SeleniumLibrary and/or AppiumLibrary. |
    | Afterwards, the session may be opened. See `Open Eyes Session`.  |
    | *Visual Checks* |
    | Between opening and closing the session, you can run your visual checks. |
    | See `Check Eyes Region`, `Check Eyes Region By Element`, `Check Eyes Region By Selector` and `Check Eyes Window` |
    | You can also verify if there's an open session with `Eyes Session Is Open` |
    | *Close Eyes Session* |
    | See `Close Eyes Session`. |

    - Example:

        | *Keywords*         |  *Parameters*                                                                                                                                                                                                                    |
        | Open Browser       |  http://google.com/ | gc                |                       
        | Open Eyes Session  |  YourApplitoolsKey  | EyesLibrary_AppName |  EyesLibrary_TestName |  
        | Check Eyes Window  |  Google Homepage            |                              
        | Close Eyes Session |  

    *Using Selectors*

    Using the keyword Check Eyes Region By Selector. *All* the following strategies are supported:

    | *Strategy*        | *Example*                                                                                                      | *Description*                                   |
    | CSS SELECTOR      | Check Eyes Region By Selector `|` .first.expanded.dropdown `|`  CssElement      `       |` CSS SELECTOR        | Matches by CSS Selector                         |
    | XPATH             | Check Eyes Region By Selector `|` //div[@id='my_element']  `|`  XpathElement   `        |` XPATH               | Matches with arbitrary XPath expression         |
    | ID                | Check Eyes Region By Selector `|` my_element               `|`  IdElement    `          |` ID                  | Matches by @id attribute                        |
    | CLASS NAME        | Check Eyes Region By Selector `|` element-search           `|`  ClassElement   `        |` CLASS NAME          | Matches by @class attribute                     |
    | LINK TEXT         | Check Eyes Region By Selector `|` My Link                  `|`  LinkTextElement      `  |` LINK TEXT           | Matches anchor elements by their link text      |
    | PARTIAL LINK TEXT | Check Eyes Region By Selector `|` My Li                    `|`  PartialLinkTextElement` |` PARTIAL LINK TEXT   | Matches anchor elements by partial link text    |
    | NAME              | Check Eyes Region By Selector `|` my_element               `|`  NameElement    `        |` NAME                | Matches by @name attribute                      |
    | TAG NAME          | Check Eyes Region By Selector `|` div                      `|`  TagNameElement       `  |` TAG NAME            | Matches by HTML tag name                        |
    """

    ROBOT_LIBRARY_SCOPE = "GLOBAL"
    ROBOT_LIBRARY_VERSION = VERSION

    def __init__(
        self,
        apikey=None,
        appname=None,
        testname=None,
        library="SeleniumLibrary",
        matchlevel=None,
        enable_eyes_log=False,
        osname=None,
        browsername=None,
    ):

        self.library_arguments = {
            "apikey": apikey,
            "appname": appname,
            "testname": testname,
            "library": library,
            "matchlevel": matchlevel,
            "enable_eyes_log": enable_eyes_log,
            "osname": osname,
            "browsername": browsername,
        }

        variables.init()




#!/usr/bin/env python

import sys

# from distutils.core import setup
from setuptools import setup
from os import path
import io
from os.path import join, dirname

sys.path.append(join(dirname(__file__), "EyesLibrary"))

execfile("EyesLibrary/version.py")

#this_directory = path.abspath(path.dirname(__file__))
with io.open('README.md', encoding='utf-8') as f:
    long_description = f.read()

# execfile(join(dirname(__file__), 'EyesLibrary', 'version.py'))

# version_file = open("EyesLibrary/version.py")
# VERSION = version_file.read().strip()

# DESCRIPTION = """
# EyesLibrary is a visual verification library for Robot Framework
# that leverages the Eyes-Selenium and Selenium/Appium libraries.
# """[
#     1:-1
# ]

setup(
    name="robotframework-eyeslibrary",
    version=VERSION,
    description="Visual Verification testing library for Robot Framework",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Joel Oliveira, Sofia Nunes",
    author_email="<joel.oliveira@criticalsoftware.com>, <sofia.nunes@criticalsoftware.com>",
    url="https://github.com/joel-oliveira/EyesLibrary",
    license="Apache License 2.0",
    keywords="robotframework testing testautomation eyes-selenium selenium appium visual-verification",
    platforms="any",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 2.7",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Framework :: Robot Framework :: Library",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Quality Assurance",
    ],
    install_requires=["robotframework >= 3.1.1", "eyes-selenium >= 3.15.2"],
    packages=["EyesLibrary"],
    data_files=[
        (
            "tests/acceptance",
            [
                "tests/acceptance/android_app.robot",
                "tests/acceptance/android_browser.robot",
                "tests/acceptance/android_hybrid_app.robot",
                "tests/acceptance/ios_app.robot",
                "tests/acceptance/ios_browser.robot",
                "tests/acceptance/ios_hybrid_app.robot",
                "tests/acceptance/web.robot",
            ],
        ),
        ("docs", ["docs/ChangeLog.txt", "docs/index.html"]),
    ],
)
